@@ -176,6 +176,16 @@ static apr_status_t clean_resp(void *data)
     return APR_SUCCESS;
 }
 
+/* cleanup for conns */
+static apr_status_t clean_conn(void *data)
+{
+    serf_connection_t *conn = data;
+
+    serf_connection_close(conn);
+
+    return APR_SUCCESS;
+}
+
 /* Update the pollset for this connection. We tweak the pollset based on
  * whether we want to read and/or write, given conditions within the
  * connection. If the connection is not (yet) in the pollset, then it
@@ -294,7 +304,13 @@ static apr_status_t open_connections(serf_context_t *ctx)
         apr_pool_clear(conn->skt_pool);
         apr_pool_cleanup_register(conn->skt_pool, conn, clean_skt, clean_skt);
 
-        if ((status = apr_socket_create(&skt, conn->address->family,
+        /* Do we have to connect to a proxy server? */
+        if (ctx->proxy_address)
+            serv_addr = ctx->proxy_address;
+        else
+            serv_addr = conn->address;
+
+        if ((status = apr_socket_create(&skt, serv_addr->family,
                                         SOCK_STREAM,
 #if APR_MAJOR_VERSION > 0
                                         APR_PROTO_TCP,
@@ -314,12 +330,6 @@ static apr_status_t open_connections(serf_context_t *ctx)
         /* Configured. Store it into the connection now. */
         conn->skt = skt;
 
-        /* Do we have to connect to a proxy server? */
-        if (ctx->proxy_address)
-            serv_addr = ctx->proxy_address;
-        else
-            serv_addr = conn->address;
-
         /* Now that the socket is set up, let's connect it. This should
          * return immediately.
          */
@@ -681,6 +691,12 @@ static apr_status_t write_to_connection(serf_connection_t *conn)
                                          &request->handler,
                                          &request->handler_baton,
                                          request->respool);
+
+            if (read_status) {
+                /* Something bad happened. Propagate any errors. */
+                return read_status;
+            }
+
             request->setup = NULL;
         }
 
@@ -726,15 +742,16 @@ static apr_status_t write_to_connection(serf_connection_t *conn)
                 return status;
         }
 
-        if (APR_STATUS_IS_EOF(read_status)) {
-            /* If we hit the end of the request bucket, then clear it out to
-             * signify that we're done sending the request. On the next
-             * iteration through this loop, we'll see if there are other
-             * requests that need to be sent ("pipelining").
+        if (APR_STATUS_IS_EOF(read_status) &&
+            conn->vec_len == 0) {
+            /* If we hit the end of the request bucket and all of its data has
+             * been written, then clear it out to signify that we're done
+             * sending the request. On the next iteration through this loop:
+             * - if there are remaining bytes they will be written, and as the 
+             * request bucket will be completely read it will be destroyed then.
+             * - we'll see if there are other requests that need to be sent 
+             * ("pipelining").
              */
-            /* ### woah. watch out for the unwritten stuff. gotta restructure
-               ### this a bit more to avoid killing a bucket where the
-               ### data is hanging out in the unwritten field. */
             serf_bucket_destroy(request->req_bkt);
             request->req_bkt = NULL;
 
@@ -1173,7 +1190,8 @@ SERF_DECLARE(serf_connection_t *) serf_connection_create(
     /* Create a subpool for our connection. */
     apr_pool_create(&conn->skt_pool, conn->pool);
 
-    /* ### register a cleanup */
+    /* register a cleanup */
+    apr_pool_cleanup_register(conn->pool, conn, clean_conn, apr_pool_cleanup_null);
 
     /* Add the connection to the context. */
     *(serf_connection_t **)apr_array_push(ctx->conns) = conn;
