@@ -420,6 +420,11 @@ serf_ssl_server_cert_callback_set(serf_ssl_context_t *context,
 SERF_DECLARE(apr_status_t)
 serf_ssl_use_default_certificates(serf_ssl_context_t *context);
 
+/**
+ * Return the depth of the certificate.
+ */
+SERF_DECLARE(int) serf_ssl_cert_depth(const serf_ssl_certificate_t *cert);
+
 /**
  * Extract the fields of the issuer in a table with keys (E, CN, OU, O, L, 
  * ST and C). The returned table will be allocated in @a pool.
@@ -441,6 +446,13 @@ serf_ssl_cert_subject(const serf_ssl_certificate_t *cert, apr_pool_t *pool);
 SERF_DECLARE(apr_hash_t *)
 serf_ssl_cert_certificate(const serf_ssl_certificate_t *cert, apr_pool_t *pool);
 
+/**
+ * Export a certificate to base64-encoded, zero-terminated string.
+ * The returned string is allocated in @a pool. Returns NULL on failure.
+ */
+SERF_DECLARE(const char *)
+serf_ssl_cert_export(const serf_ssl_certificate_t *cert, apr_pool_t *pool);
+
 /**
  * Load a CA certificate file from a path @a file_path. If the file was loaded
  * and parsed correctly, a certificate @a cert will be created and returned.
