@@ -30,7 +30,6 @@
 #include "serf.h"
 #include "serf_bucket_util.h"
 
-#define SERF_VERSION_STRING "0.01"
 /*#define SERF_VERBOSE*/
 
 #if !APR_HAS_THREADS
@@ -123,8 +122,8 @@ typedef struct {
     apr_thread_cond_t *doc_queue_condvar;
 
     const char *hostinfo;
-  
-    /* includes: path, query, fragment. */ 
+
+    /* includes: path, query, fragment. */
     char *full_path;
     apr_size_t full_path_len;
 
@@ -155,6 +154,7 @@ typedef struct {
 #define apr_atomic_inc32 apr_atomic_inc
 #define apr_atomic_dec32 apr_atomic_dec
 #define apr_atomic_read32 apr_atomic_read
+#define apr_atomic_set32 apr_atomic_set
 #endif
 
 static apr_status_t handle_response(serf_request_t *request,
@@ -563,10 +563,10 @@ static void * APR_THREAD_FUNC parser_thread(apr_thread_t *thread, void *data)
     apr_pool_create(&subpool, pool);
 
     while (1) {
-        apr_pool_clear(subpool);
-
         doc_path_t *dup;
 
+        apr_pool_clear(subpool);
+
         /* Grab it. */
         apr_thread_mutex_lock(ctx->mutex);
         /* Sleep. */
@@ -725,6 +725,7 @@ int main(int argc, const char **argv)
     handler_ctx->requests_outstanding =
         (apr_uint32_t*)serf_bucket_mem_alloc(app_ctx.bkt_alloc,
                                              sizeof(apr_uint32_t));
+    apr_atomic_set32(handler_ctx->requests_outstanding, 0);
     handler_ctx->hdr_read = 0;
 
     parser_ctx = (void*)serf_bucket_mem_alloc(app_ctx.bkt_alloc,
